{****************************************************}
{  Turbo Vision Demonstration Program                }
{  Borland Technical Support                         }
{****************************************************}

{ A specific code sample for the benefit of Borland customers
  to illustrate a particular programming technique. }

program TVScroll;

{ Demonstrates how to have scroll bars operate within a dialog box }
{ An early demo. Keyboard support has not been added. }
{$X+}

uses
  Memory, Drivers,                     { Non-OOP Vision Units }
  Objects, Views, Menus, Dialogs,      { Turbo Vision OOP Units }
  App;

const
  cm_Action = 100;
  cmMove = 101;

type
  PActionDialog = ^TActionDialog;
  TActionDialog = object(TDialog)
    HScrollBar,
    VScrollBar: PScrollBar;
    Button: PButton;
    OX, OY: Integer;
    procedure HandleEvent(var Event: TEvent); virtual;
  end;

  PMainApp = ^TMainApp;
  TMainApp = object(TApplication)
    procedure InitMenuBar; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure CMAction(var Event: TEvent); virtual;
  end;

procedure TActionDialog.HandleEvent(var Event: TEvent);
begin
  TDialog.HandleEvent(Event);
  if (Event.What = evCommand) or (Event.What = evBroadCast) then
    case Event.Command of
    cmMove:
      begin
        PutEvent(Event);
        EndModal(cmQuit);
        ClearEvent(Event);
      end;
    cmScrollBarChanged:
      begin
{ When the scrollbar changes move the controls in the dialog. (This
  operation could be done with a ForEach call to affect any number
  of controls. }
        Button^.MoveTo(OX + HScrollBar^.Value,
          OY + VScrollBar^.Value);
        ClearEvent(Event);
      end;
    end;
end;

procedure TMainApp.InitMenuBar;
var
  R: TRect;
begin
  GetExtent(R);
  R.B.Y := R.A.Y + 1;
  MenuBar := New(PMenuBar, Init(R, NewMenu(
    NewItem('~A~ction', '', kbNoKey, cm_Action, hcNoContext,
    nil)
  )));
end;

procedure TMainApp.HandleEvent(var Event: TEvent);
var
  R: TRect;
begin
  TApplication.HandleEvent(Event);
  case Event.What of
    evCommand:
      case Event.Command of
        cm_Action: CMAction(Event);
      end;
  end;
end;

procedure TMainApp.CMAction(var Event: TEvent);
var
  Dialog: PActionDialog;
  R: TRect;
  Interior: PView;
  Stuff: PGroup;
begin
  R.Assign(5, 3, 50, 20);
  New(Dialog, Init(R, 'Action Dialog'));

  { Permit the Dialog to be resizable }
  Dialog^.Flags:=Dialog^.Flags or wfGrow;
  Dialog^.GrowMode := gfGrowHiX + gfGrowHiY;
  Dialog^.Options := Dialog^.Options or ofFramed;

  { Insert scrollbars }
  Dialog^.GetExtent(R);
  R.A.Y := R.B.Y - 1;
  R.A.X := R.A.X + 10;
  R.B.X := R.B.X - 2;
  with Dialog^ do
    begin
      HScrollBar := New(PScrollBar, Init(R));
      HScrollBar^.SetRange(-40, 40);
      HScrollBar^.SetValue(0);
      Insert(HScrollBar);
    end;

  Dialog^.GetExtent(R);
  R.A.X := R.B.X - 1;
  R.A.Y := R.A.Y + 5;
  R.B.Y := R.B.Y - 1;
  with Dialog^ do
    begin
      VScrollBar := New(PScrollBar, Init(R));
      VScrollBar^.SetRange(-20, 20);
      VScrollBar^.SetValue(0);
      Insert(VScrollBar);
    end;

  { Here is established the group called stuff.  It makes up the
    visual components seen inside the dialog.  The interior
    (a simple PView) and the Button are inserted into stuff not
    the dialog.  Only Stuff and the scrollbars are inserted into
    the dialog.
  }
  Dialog^.GetExtent(R);
  inc(R.A.X); inc(R.A.Y);
  dec(R.B.X); dec(R.B.Y);

  Stuff:= New(PGroup, Init(R));
  Stuff^.GrowMode := gfGrowHiX + gfGrowHiY;
  Stuff^.Options := Stuff^.Options or ofFramed;

  Dialog^.Insert(Stuff);

  Stuff^.GetExtent(R);
  Interior := New(PView, Init(R));
  Interior^.GrowMode := gfGrowHiX + gfGrowHiY;
  Interior^.Options := Interior^.Options or ofFramed;
  Stuff^.Insert(Interior);

  R.Assign(2, 13, 12, 15);
  Dialog^.OX := 2;  {Setting the origin point for the button}
  Dialog^.OY := 13;
  Dialog^.Button := New(PButton, Init(R, '~O~k', cmMove, bfDefault));
  Stuff^.Insert(Dialog^.Button);
  Desktop^.ExecView(Dialog);
end;

var
  MainApp: TMainApp;
  Event: TEvent;
begin
  MainApp.Init;
  MainApp.Run;
  MainApp.Done;
end.

